<?php

declare(strict_types=1);

namespace Drupal\Component\OTUnit;

use Drupal\Core\StreamWrapper\PublicStream;
use OpenTelemetry\Contrib\Otlp\ContentTypes;
use OpenTelemetry\Contrib\Otlp\ProtobufSerializer;
use Opentelemetry\Proto\Collector\Logs\V1\ExportLogsServiceRequest;
use Opentelemetry\Proto\Collector\Metrics\V1\ExportMetricsServiceRequest;
use Opentelemetry\Proto\Collector\Trace\V1\ExportTraceServiceRequest;
use OpenTelemetry\SDK\Common\Configuration\Variables;
use OpenTelemetry\SDK\Common\Export\Stream\StreamTransport;

/**
 * Provides methods to setup file exporters and retrieve results.
 */
trait FileExporterTrait {

  use JsonLinesTrait;

  /**
   * File name of otlp logs file.
   */
  protected string $logsFilename = 'otlp-logs.jsonl';

  /**
   * File name of otlp metrics file.
   */
  protected string $metricsFilename = 'otlp-metrics.jsonl';

  /**
   * File name of otlp traces file.
   */
  protected string $tracesFilename = 'otlp-traces.jsonl';

  /**
   * Returns an associative array of configuration variables.
   *
   * @return array<string, string>
   *   Configuration variables as key-value pairs.
   */
  protected function getFileExporterConfigurationVariables(): array {
    return [
      Variables::OTEL_EXPORTER_OTLP_LOGS_ENDPOINT => $this->getFileExporterEndpoint($this->logsFilename),
      Variables::OTEL_EXPORTER_OTLP_METRICS_ENDPOINT => $this->getFileExporterEndpoint($this->metricsFilename),
      Variables::OTEL_EXPORTER_OTLP_TRACES_ENDPOINT => $this->getFileExporterEndpoint($this->tracesFilename),
      Variables::OTEL_LOGS_EXPORTER => 'otlp/file',
      Variables::OTEL_METRICS_EXPORTER => 'otlp/file',
      Variables::OTEL_TRACES_EXPORTER => 'otlp/file',
    ];
  }

  /**
   * Truncates result files.
   */
  protected function resetFileExporterResultFiles(): void {
    $filenames = [
      $this->tracesFilename,
      $this->metricsFilename,
      $this->logsFilename,
    ];
    foreach ($filenames as $filename) {
      $stream = fopen($this->getFileExporterEndpoint($filename), 'w');
      assert($stream !== FALSE);
      ftruncate($stream, 0);
      fclose($stream);
    }
  }

  /**
   * Read the given amount of log record lines from an NDJSON file.
   *
   * This implementation reads NDJSON formatted lines from offset up to the
   * given limit. In order to guard against race conditions when the file is
   * still being written to, up to the specified attempts are made to get the
   * data before giving up. The polling interval is increased exponentially.
   *
   * @param string|null $path
   *   Path to the log results file.
   * @param int $limit
   *   Number of lines to read.
   * @param int $offset
   *   Number of lines to skip.
   * @param int $attempts
   *   Number of attempts to make.
   *
   * @return \Opentelemetry\Proto\Collector\Logs\V1\ExportLogsServiceRequest[]
   *   Decoded metrics.
   */
  protected function waitForLogs(
    ?string $path = NULL,
    int $limit = 1,
    int $offset = 0,
    int $attempts = 8,
  ): array {
    $lines = $this->waitForJsonLines(
      path: $path ?? $this->getFileExporterEndpoint($this->logsFilename),
      limit: $limit,
      offset: $offset,
      attempts: $attempts,
    );
    $serializer = $this->getSerializer();
    return array_map(function ($line) use ($serializer) {
      $message = new ExportLogsServiceRequest();
      $serializer->hydrate($message, $line);
      return $message;
    }, $lines);
  }

  /**
   * Read the given amount of metric record lines from an NDJSON file.
   *
   * This implementation reads NDJSON formatted lines from offset up to the
   * given limit. In order to guard against race conditions when the file is
   * still being written to, up to the specified attempts are made to get the
   * data before giving up. The polling interval is increased exponentially.
   *
   * @param string|null $path
   *   Path to the metric results file.
   * @param int $limit
   *   Number of lines to read.
   * @param int $offset
   *   Number of lines to skip.
   * @param int $attempts
   *   Number of attempts to make.
   *
   * @return \Opentelemetry\Proto\Collector\Metrics\V1\ExportMetricsServiceRequest[]
   *   Decoded metrics.
   */
  protected function waitForMetrics(
    ?string $path = NULL,
    int $limit = 1,
    int $offset = 0,
    int $attempts = 8,
  ): array {
    $lines = $this->waitForJsonLines(
      path: $path ?? $this->getFileExporterEndpoint($this->metricsFilename),
      limit: $limit,
      offset: $offset,
      attempts: $attempts,
    );
    $serializer = $this->getSerializer();
    return array_map(function ($line) use ($serializer) {
      $message = new ExportMetricsServiceRequest();
      $serializer->hydrate($message, $line);
      return $message;
    }, $lines);
  }

  /**
   * Read the given amount of trace record lines from an NDJSON file.
   *
   * This implementation reads NDJSON formatted lines from offset up to the
   * given limit. In order to guard against race conditions when the file is
   * still being written to, up to the specified attempts are made to get the
   * data before giving up. The polling interval is increased exponentially.
   *
   * @param string|null $path
   *   Path to the trace results file.
   * @param int $limit
   *   Number of lines to read.
   * @param int $offset
   *   Number of lines to skip.
   * @param int $attempts
   *   Number of attempts to make.
   *
   * @return \Opentelemetry\Proto\Collector\Trace\V1\ExportTraceServiceRequest[]
   *   Decoded traces.
   */
  protected function waitForTraces(
    ?string $path = NULL,
    int $limit = 1,
    int $offset = 0,
    int $attempts = 8,
  ): array {
    $lines = $this->waitForJsonLines(
      path: $path ?? $this->getFileExporterEndpoint($this->tracesFilename),
      limit: $limit,
      offset: $offset,
      attempts: $attempts,
    );
    $serializer = $this->getSerializer();
    return array_map(function ($line) use ($serializer) {
      $message = new ExportTraceServiceRequest();
      $serializer->hydrate($message, $line);
      return $message;
    }, $lines);
  }

  /**
   * Returns the base path where otlp result files are stored.
   */
  protected function getFileExporterWorkingPath(): string {
    return realpath(PublicStream::basePath());
  }

  /**
   * Returns the endpoint uri for the given results file.
   *
   * @param string $filename
   *   The filename of a results file.
   */
  protected function getFileExporterEndpoint(string $filename): string {
    return "file://{$this->getFileExporterWorkingPath()}/{$filename}";
  }

  /**
   * Get protobuf serializer.
   */
  protected static function getSerializer(): ProtobufSerializer {
    return ProtobufSerializer::forTransport(
      new StreamTransport(NULL, ContentTypes::NDJSON)
    );
  }

}
