<?php

declare(strict_types=1);

namespace Drupal\Component\OTUnit;

/**
 * Provides methods to help with NDJSON formatted files.
 */
trait JsonLinesTrait {

  /**
   * Read the given amount of lines from an NDJSON file.
   *
   * This implementation reads NDJSON formatted lines from offset up to the
   * given limit. In order to guard against race conditions when the file is
   * still being written to, up to the specified attempts are made to get the
   * data before giving up. The polling interval is increased exponentially.
   *
   * @param string $path
   *   Path to the file. Does not need to exist yet.
   * @param int $limit
   *   Number of lines to read.
   * @param int $offset
   *   Number of lines to skip.
   * @param int $attempts
   *   Number of attempts to make.
   *
   * @return string[]
   *   Lines extracted from file.
   */
  final protected static function waitForJsonLines(
    string $path,
    int $limit = 1,
    int $offset = 0,
    int $attempts = 8,
  ): array {
    for ($attempt = 0; $attempt < $attempts; $attempt++) {
      if (file_exists($path)) {
        $data = file_get_contents($path);
        $lines = explode("\n", $data);
        if (count($lines) > $offset + $limit) {
          return array_slice($lines, $offset, $limit);
        }
      }
      usleep(1000 * (2 ** $attempt));
    }
    throw new \RuntimeException(sprintf("Failed to collect JSON lines from %s", $path));
  }

}
