<?php

declare(strict_types=1);

namespace Drupal\Component\OTUnit;

use Opentelemetry\Proto\Collector\Logs\V1\ExportLogsServiceRequest;
use Opentelemetry\Proto\Collector\Trace\V1\ExportTraceServiceRequest;
use Opentelemetry\Proto\Common\V1\AnyValue;
use Opentelemetry\Proto\Common\V1\InstrumentationScope;
use Opentelemetry\Proto\Common\V1\KeyValue;
use Opentelemetry\Proto\Logs\V1\ResourceLogs;
use Opentelemetry\Proto\Logs\V1\ScopeLogs;
use Opentelemetry\Proto\Resource\V1\Resource;
use Opentelemetry\Proto\Trace\V1\ResourceSpans;
use Opentelemetry\Proto\Trace\V1\ScopeSpans;

/**
 * Provides methods to inspect otlp protocol objects.
 */
trait ProtoHelpersTrait {

  /**
   * Looks up a key-value pair from a list of attributes.
   *
   * @param string $key
   *   The key to find in the attribute list.
   * @param Opentelemetry\Proto\Common\V1\KeyValue $attributes
   *   Attribute list.
   */
  final protected static function getAttributeValue(string $key, iterable $attributes): ?AnyValue {
    foreach ($attributes as $attribute) {
      assert($attribute instanceof KeyValue);
      if ($attribute->getKey() === $key) {
        return $attribute->getValue();
      }
    }
    return NULL;
  }

  /**
   * Extracts resource record from trace message.
   */
  final protected static function getTraceResource(ExportTraceServiceRequest $message, int $i = 0): ?Resource {
    $resourceSpans = $message->getResourceSpans();
    assert($resourceSpans[$i] instanceof ResourceSpans);
    return $resourceSpans[$i]->getResource();
  }

  /**
   * Extracts instrumentation scope from trace message.
   *
   * @return \Opentelemetry\Proto\Common\V1\InstrumentationScope|null
   *   Extracted scope.
   */
  final protected static function getTraceScope(ExportTraceServiceRequest $message, int $j = 0, int $i = 0): ?InstrumentationScope {
    $resourceSpans = $message->getResourceSpans();
    assert($resourceSpans[$i] instanceof ResourceSpans);
    $scopeSpans = $resourceSpans[$i]->getScopeSpans();
    assert($scopeSpans[$j] instanceof ScopeSpans);
    return $scopeSpans[$j]->getScope();
  }

  /**
   * Extract spans from trace message.
   *
   * @return \Opentelemetry\Proto\Trace\V1\Span[]|null
   *   Extracted spans.
   */
  final protected static function getTraceSpans(ExportTraceServiceRequest $message, int $j = 0, int $i = 0): ?iterable {
    $resourceSpans = $message->getResourceSpans();
    assert($resourceSpans[$i] instanceof ResourceSpans);
    $scopeSpans = $resourceSpans[$i]->getScopeSpans();
    assert($scopeSpans[$j] instanceof ScopeSpans);
    return $scopeSpans[$j]->getSpans();
  }

  /**
   * Extracts resource record from log message.
   */
  final protected static function getLogResource(ExportLogsServiceRequest $message, int $i = 0): ?Resource {
    $resourceLogs = $message->getResourceLogs();
    assert($resourceLogs[$i] instanceof ResourceLogs);
    return $resourceLogs[$i]->getResource();
  }

  /**
   * Extracts instrumentation scope from log message.
   *
   * @return \Opentelemetry\Proto\Common\V1\InstrumentationScope|null
   *   Extracted scope.
   */
  final protected static function getLogScope(ExportLogsServiceRequest $message, int $j = 0, int $i = 0): ?InstrumentationScope {
    $resourceLogs = $message->getResourceLogs();
    assert($resourceLogs[$i] instanceof ResourceLogs);
    $scopeLogs = $resourceLogs[$i]->getScopeLogs();
    assert($scopeLogs[$j] instanceof ScopeLogs);
    return $scopeLogs[$j]->getScope();
  }

  /**
   * Extract log records.
   *
   * @return \Opentelemetry\Proto\Logs\V1\LogRecord[]|null
   *   Extracted log records.
   */
  final protected static function getLogRecords(ExportLogsServiceRequest $message, int $j = 0, int $i = 0): ?iterable {
    $resourceLogs = $message->getResourceLogs();
    assert($resourceLogs[$i] instanceof ResourceLogs);
    $scopeLogs = $resourceLogs[$i]->getScopeLogs();
    assert($scopeLogs[$j] instanceof ScopeLogs);
    return $scopeLogs[$j]->getLogRecords();
  }

}
