<?php

declare(strict_types=1);

namespace Drupal\Component\OTUnit;

use Composer\InstalledVersions;
use Drupal\Core\StreamWrapper\PublicStream;
use PHPUnit\Framework\Attributes\After;
use PHPUnit\Framework\Attributes\Before;
use Symfony\Component\Process\Process;

/**
 * Provides methods to set up OpenTelemetry SDK Autoloading.
 *
 * Caution: the project composer.json is modified directly. Tests relying on
 * this trait must not be run in parallel.
 */
trait SdkAutoloadingTrait {

  /**
   * Path to composer.json file.
   */
  protected string $composerJsonPath = '';

  /**
   * Original content of project composer.json.
   */
  protected string $composerOriginalContent = '';

  /**
   * Record composer.json before every test.
   */
  #[Before]
  public function recordComposerJson(): void {
    $this->composerOriginalContent = $this->loadComposerJson();
  }

  /**
   * Restore composer.json after every test.
   */
  #[After]
  public function restoreComposerJson(): void {
    if ($this->composerJsonPath && $this->composerOriginalContent) {
      $this->saveComposerJson($this->composerOriginalContent);
      $this->composerJsonPath = "";
      $this->composerOriginalContent = "";
    }
  }

  /**
   * Read current composer.json.
   *
   * @return string
   *   The serialized composer.json content.
   */
  protected function loadComposerJson(): string {
    if ($this->composerJsonPath === "") {
      $rootPackage = InstalledVersions::getRootPackage();
      $rootPath = realpath($rootPackage['install_path']);
      $this->composerJsonPath = implode('/', [$rootPath, 'composer.json']);
    }

    $composerJsonContent = file_get_contents($this->composerJsonPath);
    $this->assertNotFalse($composerJsonContent);
    return $composerJsonContent;
  }

  /**
   * Store composer.json.
   *
   * @param string $content
   *   The serialized composer.json content.
   */
  protected function saveComposerJson(string $content): void {
    assert($this->composerJsonPath !== "");
    $bytes = file_put_contents($this->composerJsonPath, $content);
    $this->assertNotFalse($bytes);

    $status = Process::fromShellCommandline('composer dump-autoload', dirname($this->composerJsonPath))->run();
    $this->assertEquals(0, $status);

    $freq = intval(ini_get('opcache.revalidate_freq'));
    if ($freq !== 0) {
      $delay = $freq + 1;
      trigger_error(sprintf(
        "The ini setting opcache.revalidate_freq is set to %d. Delaying " .
        "script execution for %d seconds to avoid operating with stale " .
        "autoload files. Please set opcache.revalidate_freq to 0 to avoid " .
        "this delay.", $freq, $delay), E_USER_WARNING);
      sleep($delay);
    }
  }

  /**
   * Sets up OpenTelemetry SDK autoloading via composer.
   *
   * @param array<string, string> $server
   *   OpenTelemetry SDK configuration key-value pairs.
   */
  protected function setupSdkAutoloading(array $server): void {
    $autoloadName = '_autoload-otunit.php';
    $autoloadPath = implode('/', [PublicStream::basePath(), $autoloadName]);
    $settingLines = array_map(
      fn ($key, $value) => "\$_SERVER[\"{$key}\"] = \"{$value}\";",
      array_keys($server), array_values($server),
    );
    $autoloadContents = [
      '<?php',
      ...$settingLines,
      'if (PHP_SAPI !== "cli") {',
      '  $_SERVER["OTEL_PHP_AUTOLOAD_ENABLED"] = "true";',
      '  \OpenTelemetry\SDK\SdkAutoloader::autoload();',
      '}',
    ];
    $bytes = file_put_contents($autoloadPath, implode("\n", $autoloadContents));
    $this->assertNotFalse($bytes);

    // Replace autoload section.
    $originalContent = $this->loadComposerJson();
    $composerContent = json_decode(
      $originalContent,
      flags: JSON_OBJECT_AS_ARRAY | JSON_THROW_ON_ERROR
    );
    $this->assertIsArray($composerContent);
    $composerContent['autoload']['files'][] = realpath($autoloadPath);
    $this->saveComposerJson(json_encode(
      $composerContent,
      flags: JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_LINE_TERMINATORS | JSON_THROW_ON_ERROR
    ));
  }

}
